#ifndef _TRIE_HPP_
#define _TRIE_HPP_

#include <vector>
#include <list>
#include <stdlib.h>
#include <string.h>

using namespace std;

// ==================== Definitions for trie nodes ====================

template <class NODETYPE> class TrieNodeVA {
private:
	typedef struct _NodeIndex{
		size_t index;
		TrieNodeVA<NODETYPE> * dest;
		_NodeIndex(size_t i = 0, TrieNodeVA<NODETYPE> * d = NULL){
			index = i; dest = d;
		}
	} NodeIndex;
	vector<NodeIndex> children;
	
	typedef typename vector<NodeIndex>::iterator ChildItem;
	typename TrieNodeVA<NODETYPE>::ChildItem seek(size_t index);
	typename TrieNodeVA<NODETYPE>::ChildItem addchild(size_t index, int onlyseek);
	NODETYPE nodeval;
	
public:
	TrieNodeVA(void);
	NODETYPE & val(void);
	int link(TrieNodeVA<NODETYPE> * other, size_t index);
	int unlink(size_t index);
	TrieNodeVA<NODETYPE> * child(size_t index);
};

// ==================== Definitions for tries ====================
template <class TYPE = char> class CharTrie{
public:
	typedef TrieNodeVA<int> CharTrieNodeVA;
	
private:
	typedef enum _TrackNodeMode{tnm_insert, tnm_find} TrackNodeMode;
	list<CharTrieNodeVA *> nodes;
	int track_nodes(const TYPE *str, TrackNodeMode mode);
	int node_id;
	
public:
	CharTrie(void);
	~CharTrie();
	int insert(const TYPE *str);
	int find(const TYPE *str);
	int count(void);
	int size(void);
};

#include "trie.cpp"

#endif // _TRIE_HPP_

